<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * This script handles the report generation in batch task for a single group.
 * It will produce a group Excel worksheet report that is pushed immediately to output
 * for downloading by a batch agent. No file is stored into the system.
 * groupid must be provided.
 * This script should be sheduled in a CURL call stack or a multi_CURL parallel call.
 *
 * @package    report_trainingsessions
 * @category   report
 * @author     Valery Fremaux (valery.fremaux@gmail.com)
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

require('../../../../config.php');

ob_start();
require_once($CFG->dirroot.'/blocks/use_stats/locallib.php');
require_once($CFG->dirroot.'/report/trainingsessions/locallib.php');
require_once($CFG->dirroot.'/report/trainingsessions/pro/renderers/pdfrenderers.php');

$id = required_param('id', PARAM_INT); // The course id.
$userid = required_param('userid', PARAM_INT); // The group id.
$filename = optional_param('outputname', '', PARAM_FILE);

ini_set('memory_limit', '512M');

if (!$course = $DB->get_record('course', array('id' => $id))) {
    // Do NOT print_error here as we are a document writer.
    die ('Invalid course ID');
}
$context = context_course::instance($course->id);

if (!$user = $DB->get_record('user', array('id' => $userid))) {
    // Do NOT print_error here as we are a document writer.
    die ('Invalid user ID');
}

$input = report_trainingsessions_batch_input($course);

// Security.

report_trainingsessions_back_office_access($course);

$config = get_config('report_trainingsessions');
if (!empty($config->showseconds)) {
    $durationformat = 'htmlds';
} else {
    $durationformat = 'htmld';
}

$coursestructure = report_trainingsessions_get_course_structure($course->id, $items);

// generate PDF.
$pdf = new TCPDF('P', 'mm', 'A4', true, 'UTF-8', false);

$pdf->SetTitle(get_string('sessionreportdoctitle', 'report_trainingsessions'));
$pdf->setPrintHeader(false);
$pdf->setPrintFooter(false);
$pdf->SetFontSize(9);
$pdf->setCellPaddings(2, 2, 2, 2);
$pdf->SetAutoPageBreak(false, 0);

// Sending HTTP headers.

// Define cells params.
$table = new html_table();

$head1bgcolor = $config->head1bgcolor;
$head1txtcolor = $config->head1textcolor;
$head2bgcolor = $config->head2bgcolor;
$head2txtcolor = $config->head2textcolor;
$head3bgcolor = $config->head3bgcolor;
$head3txtcolor = $config->head3textcolor;

if (!empty($config->showhits)) {
    $table->pdfhead1 = array('',
                             get_string('firstaccess'),
                             get_string('elapsed', 'report_trainingsessions'),
                             get_string('hits', 'report_trainingsessions'));
    $table->pdfsize1 = array('50%', '20%', '20%', '10%');
    $table->pdfalign1 = array('L', 'L', 'R', 'R');
} else {
    $table->pdfhead1 = array('',
                             get_string('firstaccess'),
                             get_string('elapsed', 'report_trainingsessions'));
    $table->pdfsize1 = array('50%', '25%', '25%');
    $table->pdfalign1 = array('L', 'L', 'R');
}
$table->pdfbgcolor1 = array($head1bgcolor, $head1bgcolor, $head1bgcolor);
$table->pdfcolor1 = array($head1txtcolor, $head1txtcolor, $head1txtcolor);

$table->pdfalign2 = array('L', 'L', 'L');
$table->pdfsize2 = array('50%', '25%', '25%');
$table->pdfbgcolor2 = array('#808080', $head2bgcolor, $head2bgcolor);
$table->pdfcolor2 = array($head2txtcolor, $head2txtcolor, $head2txtcolor);

if (!empty($config->showhits)) {
    $table->pdfbgcolor = array('#808080', $head2bgcolor, '#ffffff', '#ffffff');
    $table->pdfcolor = array($head2txtcolor, $head2txtcolor, '#000000', '#000000');
} else {
    $table->pdfbgcolor = array('#808080', $head2bgcolor, '#ffffff');
    $table->pdfcolor = array($head2txtcolor, $head2txtcolor, '#000000');
}

$table->pdfbgcolor3 = array($head3bgcolor, $head3bgcolor, $head3bgcolor);
$table->pdfcolor3 = array($head3txtcolor, $head3txtcolor, $head3txtcolor);
$table->pdfalign3 = array('L', 'L', 'R', 'R');

$table->pdfprintinfo = array(1, 1, 1, 1);

$pdf->AddPage();

// Define variables.
// Portrait.
$x = 20;
$y = $config->pdfabsoluteverticaloffset;
$lineincr = 8;
$dblelineincr = $lineincr * 2;
$smalllineincr = 4;

// Set alpha to no-transparency.
// $pdf->SetAlpha(1);

// Add images and lines.
report_trainingsessions_draw_frame($pdf);

// Add images and lines.
report_trainingsessions_print_header($pdf);

// Add images and lines.
report_trainingsessions_print_footer($pdf);

$logs = use_stats_extract_logs($input->from, $input->to, $user->id, $course->id);
$aggregate = use_stats_aggregate_logs($logs, 'module', 0, $input->from, $input->to);
$aggregatelastweek = use_stats_aggregate_logs($logs, 'module', 0, $input->to - WEEKSECS, $input->to);

$grantotal = report_trainingsessions_calculate_course_structure($coursestructure, $aggregate, $done, $items);

$grantotal->activityelapsed = 0 + @$aggregate['activities'][$id]->elapsed;
$grantotal->otherelapsed = 0 + @$aggregate['other'][$id]->elapsed;
$grantotal->courseelapsed = 0 + @$aggregate['course'][$id]->elapsed;

$grantotal->elapsed = 0 + @$aggregate['coursetotal'][$id]->elapsed;
$grantotal->elapsedlastweek = 0 + @$aggregatelastweek['coursetotal'][$id]->elapsed;
$grantotal->extelapsed = 0 + @$aggregate['coursetotal'][$id]->elapsed + @$aggregate['coursetotal'][0]->elapsed;
$grantotal->extelapsed += @$aggregate['coursetotal'][SITEID]->elapsed;
$grantotal->extelapsedlastweek = 0 + @$aggregatelastweek['coursetotal'][$id]->elapsed + @$aggregatelastweek['coursetotal'][0]->elapsed;
$grantotal->extelapsedlastweek += @$aggregatelastweek['coursetotal'][SITEID]->elapsed;
$grantotal->extother = 0 + @$aggregate['coursetotal'][0]->elapsed + @$aggregate['coursetotal'][SITEID]->elapsed;
$grantotal->extotherlastweek = 0 + @$aggregatelastweek['coursetotal'][0]->elapsed + @$aggregatelastweek['coursetotal'][SITEID]->elapsed;

$grantotal->activityevents = 0 + @$aggregate['activities'][$id]->events;
$grantotal->otherevents = 0 + @$aggregate['other'][$id]->events;
$grantotal->courseevents = 0 + @$aggregate['course'][$id]->events;
$grantotal->events = 0 + $grantotal->activityevents + $grantotal->otherevents + $grantotal->courseevents;
$grantotal->exteevents = 0 + @$aggregate['coursetotal'][$id]->events + @$aggregate['coursetotal'][0]->events;
$grantotal->extevents += @$aggregate['coursetotal'][SITEID]->events;

$y = report_trainingsessions_print_userinfo($pdf, $y, $user, $course, $input->from, $input->to, $grantotal, $config->recipient);
$y = report_trainingsessions_print_overheadline($pdf, $y, $table);
report_trainingsessions_print_course_structure($pdf, $y, $coursestructure, $aggregate, $table);
$dataline = array();

if (!empty($config->showhits)) {
    $dataline[] = "$done / $items ".get_string('done', 'report_trainingsessions');
} else {
    $dataline[] = '';
}
$dataline[] = report_trainingsessions_format_time($grantotal->elapsed, $durationformat);
if (!empty($config->showhits)) {
    $dataline->events = $grantotal->events;
}
// Ensure cells are sized as data columns.
if (!empty($config->showhits)) {
    $table->pdfsize2 = array('50%', '25%', '25%');
} else {
    $table->pdfsize2 = array('75%', '25%');
}

$y += 5; // Small adjustment.
$y = report_trainingsessions_print_sumline($pdf, $y, $dataline, $table);

// Sending HTTP headers.
ob_end_clean();
$loadmode = (empty($filename)) ? 'S' : 'D';
$result = $pdf->Output($filename, $loadmode);
if ($loadmode == 'S') {
    echo $result;
}
exit(0);
// echo '200';